unit EmailServer;

interface

uses
  Windows, Classes, ComObj, MailServer_TLB, SysUtils, ObjectPooler_osi, StdVcl,
  IdSMTP, IdFTP;

type
  { Custom exceptions }
  EEMailServer=class(Exception);

  { TServer }
  TServer=class
  private
    { Private variables }
    FTPPooler:TObjectPooler_osi;
    IniFilename:string;
    RefCount:Integer;
    SMTPPooler:TObjectPooler_osi; //You could add additional TObjectPooler_osi variables, rename the PooledObjectCreate()/Destroy() methods to PooledSMTPCreate()/Destroy(), and run multiple pools from within this TServer (e.g., to have a pool of email receiving objects, etc.).
    SMTPHost:string;
    SMTPUsername:string;
    SMTPPassword:string;

    { Private event handlers }
    procedure FTPPooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
    procedure FTPPooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
    procedure SMTPPooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
    procedure SMTPPooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
  protected
    { Constructor/Destructor }
    constructor Create;
    destructor Destroy;override;

    { Protected methods }
    function AutostartServer:Boolean;
    procedure SaveAutostartSettings(const Autostart:Boolean);
    function StartFTPPooler(const PoolSize, QueueLimit:Integer):Boolean;
    procedure StopFTPPooler;
    function StartSMTPPooler(const PoolSize, QueueLimit:Integer):Boolean;
    procedure StopSMTPPooler;
  end; { TServer }

  { TEmailServer }
  TEmailServer = class(TTypedComObject, IEmailServer)
  protected
    function Send(MaxWait: Integer; const ToList, CCList, BCCList: WideString;
      MaxBCCLength: Integer; const From, ReplyToList, ReceiptRecipient,
      Agent, Subject: WideString; IsHTML: WordBool;
      const Message: WideString; out OutValue: WideString): HResult;
      stdcall;
    function FTPConnect(MaxWait: Integer; const Hostname, Username, Password,
      Mode: WideString; out GrabID, ObjectIndex: Integer;
      out OutValue: WideString): HResult; stdcall;
    function FTPDisconnect(GrabID, ObjectIndex: Integer;
      out OutValue: WideString): HResult; stdcall;
    function FTPChangeDir(GrabID, ObjectIndex: Integer;
      const Directory: WideString; out OutValue: WideString): HResult;
      stdcall;
    function FTPGet(GrabID, ObjectIndex: Integer; const Mode, SourceDir,
      SourceFile, DestFile: WideString; CanOverwrite: WordBool;
      out OutValue: WideString): HResult; stdcall;
    function FTPPut(GrabID, ObjectIndex: Integer; const Mode, SourceFile,
      DestDir, DestFile: WideString; Append: WordBool;
      out OutValue: WideString): HResult; stdcall;
    function FTPDelete(GrabID, ObjectIndex: Integer; const DirName,
      Filename: WideString; out OutValue: WideString): HResult; stdcall;
    function FTPFileExists(GrabID, ObjectIndex: Integer; const DirName,
      FileName: WideString; out Exists: WordBool;
      out OutValue: WideString): HResult; stdcall;
  end; { TEmailServer }

implementation

uses
  ComServ, Forms, IdMessage;

const
  COM_ERROR_BASE = -$200;          //Error base # for user-defined errors from a COM object.
  MAILERROR = COM_ERROR_BASE;      //Error code indicating a mail error (same code for all errors--error text is loaded into the OutValue param).
  MAX_BCC_LENGTH = 10;

var
  Server:TServer;
  ExceptionList:TStringList;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                   TServer                                    }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TServer.Create;
begin { Create }
  inherited Create;
  { Init variables }
  IniFilename:=ExtractFilePath(Application.ExeName)+'EmailServer.ini';
  RefCount:=0;
  FTPPooler:=nil;
  SMTPPooler:=nil;
end; { Create }

destructor TServer.Destroy;
begin { Destroy }
  SMTPPooler.Free;
  SMTPPooler:=nil;
  FTPPooler.Free;
  FTPPooler:=nil;
  inherited Destroy;
end; { Destroy }

{==============================================================================}
{                            Private Event handlers                            }
{==============================================================================}
procedure TServer.FTPPooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
var
  NewPooledFTP:TIdFTP;
begin { FTPPooledObjectCreate }
  NewPooledFTP:=TIdFTP.Create(Sender); //Assigning Sender as the owner of the TIdFTP will ensure that if anything happens to the object pooler, all of the TIdFTP will be destroyed.
  NewPooledFTP.Passive:=True; // Make FTP trasfer passive
  PooledObject:=NewPooledFTP;
end; { FTPPooledObjectCreate }

procedure TServer.FTPPooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
begin { FTPPooledObjectDestroy }
  TIdFTP(PooledObject).Free;
  PooledObject:=nil;
end; { FTPPooledObjectDestroy }

procedure TServer.SMTPPooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
var
  NewPooledSMTP:TIdSMTP;
begin { SMTPPooledObjectCreate }
  NewPooledSMTP:=TIdSMTP.Create(Sender); //Assigning Sender as the owner of the TIdSMTP will ensure that if anything happens to the object pooler, all of the TIdSMTP will be destroyed.
  With NewPooledSMTP Do
    Begin
      Host:=SMTPHost;
      If Trim(SMTPUsername)<>'' Then
        Begin
          AuthenticationType:=atLogin;
          UserID:=SMTPUsername;
          Password:=SMTPPassword;
        End
      Else
        AuthenticationType:=atNone;
    End;
  PooledObject:=NewPooledSMTP;
end; { SMTPPooledObjectCreate }

procedure TServer.SMTPPooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
begin { SMTPPooledObjectDestroy }
  TIdSMTP(PooledObject).Free;
  PooledObject:=nil;
end; { SMTPPooledObjectDestroy }

{==============================================================================}
{                                Public methods                                }
{==============================================================================}
function TServer.AutostartServer:Boolean;
var
  IniList:TStringList;
  AutoStart:String;
  PoolSizeFTP, QueueLimitFTP, PoolSizeSMTP, QueueLimitSMTP:Integer;
begin { AutostartServer }
  Result:=False;
  { Load .ini settings }
  IniList:=TStringList.Create;
  Try
    IniList.LoadFromFile(IniFilename);
    AutoStart:=Trim(UpperCase(IniList.Values['autostart']));
    PoolSizeFTP:=StrToInt(Trim(IniList.Values['poolsize_ftp']));
    QueueLimitFTP:=StrToInt(Trim(IniList.Values['queuelimit_ftp']));
    PoolSizeSMTP:=StrToInt(Trim(IniList.Values['poolsize_smtp']));
    QueueLimitSMTP:=StrToInt(Trim(IniList.Values['queuelimit_smtp']));
    SMTPHost:=Trim(IniList.Values['smtphost']);
    SMTPUsername:=Trim(IniList.Values['smtpusername']);
    SMTPPassword:=Trim(IniList.Values['smtppassword']);
    If AutoStart<>'TRUE' Then
      Exit;
    { Start the servers }
    If not((PoolSizeFTP<1) or (QueuelimitFTP<-1)) Then
     Result:=StartFTPPooler(PoolSizeFTP,QueueLimitFTP);
    If not((PoolSizeSMTP<1) or (QueuelimitSMTP<-1)) Then
     Result:=StartSMTPPooler(PoolSizeSMTP,QueueLimitSMTP);
  Finally
    IniList.Free;
  End;
end; { AutostartServer }

procedure TServer.SaveAutostartSettings(const Autostart:Boolean);
var
  IniList:TStringList;
begin { SaveAutostartSettings }
  IniList:=TStringList.Create;
  Try
    If not(AutoStart) Then
      Begin
        { Turn off AutoStart, but keep the rest of the settings }
        IniList.LoadFromFile(IniFilename);
        IniList.Values['autostart']:='false';
      End
    Else
      Begin
        IniList.Add('autostart=true');
        If Assigned(FTPPooler) Then
          Begin
            IniList.Add('poolsize_ftp='+IntToStr(FTPPooler.PoolSize));
            IniList.Add('queuelimit_ftp='+IntToStr(FTPPooler.QueueLimit));
          End;
        If Assigned(SMTPPooler) Then
          Begin
            IniList.Add('poolsize_smtp='+IntToStr(SMTPPooler.PoolSize));
            IniList.Add('queuelimit_smtp='+IntToStr(SMTPPooler.QueueLimit));
            IniList.Add('smtphost='+SMTPHost);
            IniList.Add('smtpusername='+SMTPUsername);
            IniList.Add('smtppassword='+SMTPPassword);
          End;
      End;
    IniList.SaveToFile(IniFilename);
  Finally
    IniList.Free;
  End;
end; { SaveAutostartSettings }

function TServer.StartFTPPooler(const PoolSize, QueueLimit:Integer):Boolean;
begin { StartFTPPooler }
  Result:=False;
  If not(Assigned(FTPPooler)) Then
    Begin
      FTPPooler:=TObjectPooler_osi.Create(nil);
      Try
        FTPPooler.PoolSize:=PoolSize;
        FTPPooler.QueueLimit:=QueueLimit;
        FTPPooler.OnPooledObjectCreate:=FTPPooledObjectCreate;
        FTPPooler.OnPooledObjectDestroy:=FTPPooledObjectDestroy;
        FTPPooler.Enabled:=True;
        SaveAutostartSettings(True);
        Result:=True;
      Except
        FTPPooler.Free;
        FTPPooler:=nil;
        raise;
      End;
    End;
  //Don't need reference counting because the server should get started once and left alone.
  //Inc(RefCount);
end; { StartFTPPooler }

procedure TServer.StopFTPPooler;
begin { StopZPooler }
  //Don't need reference counting because the server should get started once and left alone.
  //If RefCount=1 Then
    //Begin
      FTPPooler.Free;
      FTPPooler:=nil;
    //End;
  //Dec(RefCount);
end; { StopFTPPooler }

function TServer.StartSMTPPooler(const PoolSize, QueueLimit:Integer):Boolean;
begin { StartSMTPPooler }
  Result:=False;
  If not(Assigned(SMTPPooler)) Then
    Begin
      SMTPPooler:=TObjectPooler_osi.Create(nil);
      Try
        SMTPPooler.PoolSize:=PoolSize;
        SMTPPooler.QueueLimit:=QueueLimit;
        SMTPPooler.OnPooledObjectCreate:=SMTPPooledObjectCreate;
        SMTPPooler.OnPooledObjectDestroy:=SMTPPooledObjectDestroy;
        SMTPPooler.Enabled:=True;
        SaveAutostartSettings(True);
        Result:=True;
      Except
        SMTPPooler.Free;
        SMTPPooler:=nil;
        raise;
      End;
    End;
  //Don't need reference counting because the server should get started once and left alone.
  //Inc(RefCount);
end; { StartSMTPPooler }

procedure TServer.StopSMTPPooler;
begin { StopZPooler }
  //Don't need reference counting because the server should get started once and left alone.
  //If RefCount=1 Then
    //Begin
      SMTPPooler.Free;
      SMTPPooler:=nil;
    //End;
  //Dec(RefCount);
end; { StopSMTPPooler }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                            TAtttitudeResortsServer                           }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
function TEmailServer.Send(MaxWait: Integer; const ToList, CCList,
  BCCList: WideString; MaxBCCLength: Integer; const From, ReplyToList,
  ReceiptRecipient, Agent, Subject: WideString; IsHTML: WordBool;
  const Message: WideString; out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
  CurrMsg:TIdMessage;
  BCCStrings:TStringList;
  i,j:Integer;
  BCCStr:String;
begin { Send }
  Try
    Result:=0;
    OutValue:='';
    CurrMsg:=nil; //Initialize this bugger so CurrMsg.Free() won't cause an access violation if CurrMsg.Create() fails (and to avoid the nested protected blocks).
    BCCStrings:=nil;
    { Grab an SMTP object }
    ObjRef:=Server.SMTPPooler.GrabObject(MaxWait);
    If ObjRef.GrabID<0 Then
      raise EEMailServer.Create('EMailServer: '+Server.SMTPPooler.ParseError(ObjRef.GrabID));
    Try
      try
        CurrMsg:=TIdMessage.Create(nil);
        BCCStrings:=TStringList.Create;
        { Expecting CRLF-delimited addresses for all fields except from and rr. }
        CurrMsg.Recipients.EMailAddresses:=StringReplace(Trim(ToList),#13#10,',',[rfReplaceAll]);
        CurrMsg.CCList.EMailAddresses:=StringReplace(Trim(CCList),#13#10,',',[rfReplaceAll]);
        CurrMsg.From.Text:=Trim(From); //Indy is not designed to allow multiple from addresses.  If you need to use multiple from addresses, send them in this format: from_user1@domain.com, from_user2@domain.com, ...
        CurrMsg.ReplyTo.EMailAddresses:=StringReplace(Trim(ReplyToList),#13#10,',',[rfReplaceAll]);
        CurrMsg.ReceiptRecipient.Text:=Trim(ReceiptRecipient); //Indy is not designed to allow multiple rr addresses.  If you need to use multiple rr addresses, send them in this format: rr_user1@domain.com, rr_user2@domain.com, ...
        CurrMsg.Subject:=Subject;
        if (IsHTML) then
          CurrMsg.ContentType:='text/html; charset=us-ascii';
        CurrMsg.Body.Text:=Message;
        If not(IsHTML) Then
          CurrMsg.Body.Text:=StringReplace(CurrMsg.Body.Text,'&amp;','&',[rfReplaceAll]);
        TIdSMTP(Server.SMTPPooler.Objects[ObjRef]).MailAgent:=Agent;
        TIdSMTP(Server.SMTPPooler.Objects[ObjRef]).Connect;
        try
          BCCStrings.Text:=BCCList;
          i:=0;
          while (i<BCCStrings.Count) do begin
            BCCStr:='';
            j:=0;
            { Add BCC names:  MaxBCCLength < 0 means no limit to the number of BCC names; MaxBCCLength = 0 means use the MAX_BCC_LENGTH default; MaxBCCLength = n (where n > 0) sets the max. BCC names to n }
            If MaxBCCLength=0 Then //If using the default max. BCC length Then...
              MaxBCCLength:=MAX_BCC_LENGTH;
            while ((MaxBCCLength<0) or (j<MaxBCCLength)) and (i<BCCStrings.Count) do begin
              if (j>0) then BCCStr:=BCCStr+',';
              BCCStr:=BCCStr+BCCStrings[i];
              Inc(j);
              Inc(i);
            end;
            CurrMsg.BccList.EMailAddresses:=BCCStr;
            TIdSMTP(Server.SMTPPooler.Objects[ObjRef]).Send(CurrMsg);
          end;
          If i=0 Then //If there are no BCC addresses, we still need to send the email...
            TIdSMTP(Server.SMTPPooler.Objects[ObjRef]).Send(CurrMsg);
        finally
          TIdSMTP(Server.SMTPPooler.Objects[ObjRef]).Disconnect;
        end;
      finally
        CurrMsg.Free;
        BCCStrings.Free;
      end;
    Finally
      Server.SMTPPooler.ReleaseObject(ObjRef);
    End;
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.Send): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  end;
end; { Send }

function TEmailServer.FTPConnect(MaxWait: Integer; const Hostname,
  Username, Password, Mode: WideString; out GrabID, ObjectIndex: Integer;
  out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPConnect }
  Try
    Result:=0;
    OutValue:='';
    { Grab an FTP object }
    ObjRef:=Server.FTPPooler.GrabObject(MaxWait);
    If ObjRef.GrabID<0 Then
      raise EEMailServer.Create('EMailServer: '+Server.FTPPooler.ParseError(ObjRef.GrabID));
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Host:=Hostname;
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).User:=Username;
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Password:=Password;
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Connect(True);
    GrabID:=ObjRef.GrabID;
    ObjectIndex:=ObjRef.ObjectIndex;
    { Set mode }
    If (AnsiCompareText(Mode,'a')=0) or (AnsiCompareText(Mode,'asc')=0) or (AnsiCompareText(Mode,'ascii')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftAscii
    Else If (AnsiCompareText(Mode,'b')=0) or (AnsiCompareText(Mode,'bin')=0) or (AnsiCompareText(Mode,'binary')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftBinary;
  Except
    On E:Exception Do
      Begin
        Server.FTPPooler.ReleaseObject(ObjRef);
        GrabID:=-1;
        ObjectIndex:=-1;
        OutValue:='Exception (TEmailServer.FTPConnect): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  end;
end; { FTPConnect }

function TEmailServer.FTPDisconnect(GrabID, ObjectIndex: Integer;
  out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPDisconnect }
  Try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Disconnect;
    Server.FTPPooler.ReleaseObject(ObjRef);
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.FTPDisconnect): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  End;
end; { FTPDisconnect }

function TEmailServer.FTPChangeDir(GrabID, ObjectIndex: Integer;
  const Directory: WideString; out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPChangeDir }
  Try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).ChangeDir(Directory);
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.FTPChangeDir): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  End;
end; { FTPChangeDir }

function TEmailServer.FTPGet(GrabID, ObjectIndex: Integer; const Mode,
  SourceDir, SourceFile, DestFile: WideString; CanOverwrite: WordBool;
  out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPGet }
  Try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    { Set mode }
    If (AnsiCompareText(Mode,'a')=0) or (AnsiCompareText(Mode,'asc')=0) or (AnsiCompareText(Mode,'ascii')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftAscii
    Else If (AnsiCompareText(Mode,'b')=0) or (AnsiCompareText(Mode,'bin')=0) or (AnsiCompareText(Mode,'binary')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftBinary;
    { Change dir, if necessary }
    If SourceDir<>'' Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).ChangeDir(SourceDir);
    { Get file }
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Get(SourceFile,DestFile,CanOverwrite);
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.FTPGet): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  End;
end; { FTPGet }

function TEmailServer.FTPPut(GrabID, ObjectIndex: Integer; const Mode,
  SourceFile, DestDir, DestFile: WideString; Append: WordBool;
  out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPPut }
  Try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    { Set mode }
    If (AnsiCompareText(Mode,'a')=0) or (AnsiCompareText(Mode,'asc')=0) or (AnsiCompareText(Mode,'ascii')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftAscii
    Else If (AnsiCompareText(Mode,'b')=0) or (AnsiCompareText(Mode,'bin')=0) or (AnsiCompareText(Mode,'binary')=0) Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).TransferType:=ftBinary;
    { Change dir, if necessary }
    If DestDir<>'' Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).ChangeDir(DestDir);
    { Put file }
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Put(SourceFile,DestFile,Append);
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.FTPPut): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  End;
end; { FTPPut }

function TEmailServer.FTPDelete(GrabID, ObjectIndex: Integer;
  const DirName, Filename: WideString; out OutValue: WideString): HResult;
var
  ObjRef:TPooledObjectRef;
begin { FTPPut }
  Try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    { Change dir, if necessary }
    If DirName<>'' Then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).ChangeDir(DirName);
    { Delete file }
    TIdFTP(Server.FTPPooler.Objects[ObjRef]).Delete(Filename);
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TEmailServer.FTPDelete): '+e.Message;
        Result:=COM_ERROR_BASE;
      End;
  End;
end; { FTPDelete }

function TEmailServer.FTPFileExists(GrabID, ObjectIndex: Integer;
  const DirName, FileName: WideString; out Exists: WordBool;
  out OutValue: WideString): HResult;
var
  ObjRef: TPooledObjectRef;
  FileList: TStringList;
begin { FTPFileExists }
  try
    Result:=0;
    OutValue:='';
    ObjRef.GrabID:=GrabID;
    ObjRef.ObjectIndex:=ObjectIndex;
    { Change dir, if necessary }
    if DirName <> '' then
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).ChangeDir(DirName);
    { Delete file }
    FileList := TStringList.Create();
    try
      TIdFTP(Server.FTPPooler.Objects[ObjRef]).List(FileList, '*.*', False);
      Exists := FileList.IndexOf(Filename) >= 0;
    finally
      FileList.Free();
    end;
  except
    on E: Exception do
    begin
      OutValue := 'Exception (TEmailServer.FTPFileExists): ' + E.Message;
      Result := COM_ERROR_BASE;
    end;
  end;
end; { FTPFileExists }

initialization
{==============================================================================}
{                                Initialization                                }
{==============================================================================}
  Try
    { Create TServer object }
    Server:=TServer.Create;
    If not(Server.AutostartServer) Then
      raise EEmailServer.Create('Server refused to autostart.');
    { Class factory call }
    TTypedComObjectFactory.Create(ComServer, TEmailServer, Class_EmailServer,
      ciMultiInstance, tmFree);
  Except
    On E:Exception Do
      Begin
        ExceptionList:=TStringList.Create;
        Try
          ExceptionList.Add(DateTimeToStr(Now));
          ExceptionList.Add('Unable to autostart EmailServer: '+E.Message);
          ExceptionList.SaveToFile(ExtractFilePath(Application.ExeName)+'emailserver_startup_failure.txt');
        Finally
          ExceptionList.Free;
        End;
        raise;
      End;
  End;

finalization
{==============================================================================}
{                                 Finalization                                 }
{==============================================================================}
  Server.Free;

end.
