unit XSLTransformServer;

interface

uses
  Classes, ComObj, XSLTServer_TLB, ObjectPooler_osi, SysUtils, Forms, ActiveX, StdVcl;

type
  { Custom exceptions }
  EXSLTransformServer = class(Exception);

  { TPooledXSLT }
  TPooledXSLT = class
  private
    Owner:TComponent;
  protected
    { Public properties }
    XMLObj:Variant;
    XSLObj:Variant;

    { Constructor/Destructor }
    constructor Create(AOwner:TComponent);
    destructor Destroy; override;
  end; { TPooledXSLT }

  { TServer }
  TServer = class
  private
    { Private variables }
    IniFilename:String;
    MSXMLVersion:String;
    RefCount:Integer;
    XSLTPooler:TObjectPooler_osi;

    { Private event handlers }
    procedure PooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
    procedure PooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
  protected
    { Constructor/Destructor }
    constructor Create;
    destructor Destroy; override;

    { Protected methods }
    function AutostartServer:Boolean;
    procedure SaveAutostartSettings(const Autostart:Boolean);
    function StartXSLTPooler(const PoolSize, QueueLimit:Integer):Boolean;
    procedure StopXSLTPooler;
  end; { TServer }

  { TXSLTransformServer }
  TXSLTransformServer = class(TTypedComObject, IXSLTransformServer)
  protected
    function QueryServerParams(out OutValue: WideString): HResult; stdcall;
    function Transform(MaxWait: Integer; const XML, XSLT: WideString;
      out OutValue: WideString): HResult; stdcall;
  end; { TXSLTransformServer }

implementation

uses
  ComServ;

const
  COM_ERROR_BASE = -$200;               //Error base # for user-defined errors from a COM object.
  XSLTERROR = COM_ERROR_BASE;           //Error code indicating an XSLTransform error (same code for all errors--error text is loaded into the OutValue param).

var
  Server:TServer;
  ExceptionList:TStringList;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                TPooledXSLT                                   }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TPooledXSLT.Create(AOwner:TComponent);
begin { Create }
  inherited Create;
  Owner:=AOwner;
  XMLObj:=CreateOleObject('MSXML2.DOMDocument.'+Server.MSXMLVersion+'.0');
  XMLObj.Async:=False;
  If StrToInt(Server.MSXMLVersion)>3 Then
    Begin
      XMLObj.setProperty('NewParser','True');
      XMLObj.setProperty('SelectionNamespaces','xmlns:xsl="http://www.w3.org/1999/XSL/Transform"');
    End;
  XMLObj.ValidateOnParse:=False;
  XSLObj:=CreateOleObject('MSXML2.DOMDocument.'+Server.MSXMLVersion+'.0');
  XSLObj.Async:=False;
  If StrToInt(Server.MSXMLVersion)>3 Then
    Begin
      XSLObj.setProperty('NewParser','True');
      XSLObj.setProperty('SelectionNamespaces','xmlns:xsl="http://www.w3.org/1999/XSL/Transform"');
    End;
  XSLObj.ValidateOnParse:=False;
end; { Create }

destructor TPooledXSLT.Destroy;
begin { Destroy }
  XSLObj:=unassigned;
  XMLObj:=unassigned;
  inherited Destroy;
end; { Destroy }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                   TServer                                    }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TServer.Create;
begin { Create }
  inherited Create;
  { Init variables }
  IniFilename:=ExtractFilePath(Application.ExeName)+'XSLTransformServer.ini';
  RefCount:=0;
  XSLTPooler:=nil;
end; { Create }

destructor TServer.Destroy;
begin { Destroy }
  XSLTPooler.Free;
  XSLTPooler:=nil;
  inherited Destroy;
end; { Destroy }

{==============================================================================}
{                            Private Event handlers                            }
{==============================================================================}
procedure TServer.PooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
var
  NewPooledXSLT:TPooledXSLT;
begin { PooledObjectCreate }
  NewPooledXSLT:=TPooledXSLT.Create(Sender); //Assigning Sender as the owner of the TPooledXSLT will ensure that if anything happens to the object pooler, all of the XSLT stuff will be destroyed.
  PooledObject:=NewPooledXSLT;
end; { PooledObjectCreate }

procedure TServer.PooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
begin { PooledObjectDestroy }
  TPooledXSLT(PooledObject).Free;
  PooledObject:=nil;
end; { PooledObjectDestroy }

{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
function TServer.AutostartServer:Boolean;
var
  IniList:TStringList;
  AutoStart:String;
  PoolSize, QueueLimit:Integer;
begin { AutostartServer }
  Result:=False;
  { Load .ini settings }
  IniList:=TStringList.Create;
  Try
    IniList.LoadFromFile(IniFilename);
    AutoStart:=Trim(UpperCase(IniList.Values['autostart']));
    PoolSize:=StrToInt(Trim(IniList.Values['poolsize']));
    QueueLimit:=StrToInt(Trim(IniList.Values['queuelimit']));
    MSXMLVersion:=Trim(UpperCase(IniList.Values['msxmlversion']));
    If (AutoStart<>'TRUE') or (PoolSize<1) or (Queuelimit<-1) or (StrToInt(MSXMLVersion)<3) Then
      Exit;
    { Start the server }
    Result:=StartXSLTPooler(PoolSize,QueueLimit);
  Finally
    IniList.Free;
  End;
end; { AutostartServer }

procedure TServer.SaveAutostartSettings(const Autostart:Boolean);
var
  IniList:TStringList;
begin { SaveAutostartSettings }
  IniList:=TStringList.Create;
  Try
    If not(AutoStart) Then
      Begin
        { Turn off AutoStart, but keep the rest of the settings }
        IniList.LoadFromFile(IniFilename);
        IniList.Values['autostart']:='false';
      End
    Else
      Begin
        IniList.Add('autostart=true');
        IniList.Add('poolsize='+IntToStr(XSLTPooler.PoolSize));
        IniList.Add('queuelimit='+IntToStr(XSLTPooler.QueueLimit));
        IniList.Add('msxmlversion='+MSXMLVersion);
      End;
    IniList.SaveToFile(IniFilename);
  Finally
    IniList.Free;
  End;
end; { SaveAutostartSettings }

function TServer.StartXSLTPooler(const PoolSize, QueueLimit:Integer):Boolean;
begin { StartXSLTPooler }
  Result:=False;
  If not(Assigned(XSLTPooler)) Then
    Begin
      XSLTPooler:=TObjectPooler_osi.Create(nil);
      Try
        XSLTPooler.PoolSize:=PoolSize;
        XSLTPooler.QueueLimit:=QueueLimit;
        XSLTPooler.OnPooledObjectCreate:=PooledObjectCreate;
        XSLTPooler.OnPooledObjectDestroy:=PooledObjectDestroy;
        XSLTPooler.Enabled:=True;
        SaveAutostartSettings(True);
        Result:=True;
      Except
        XSLTPooler.Free;
        XSLTPooler:=nil;
        raise;
      End;
    End;
  //Don't need reference counting because the server should get started once and left alone.
  //Inc(RefCount);
end; { StartXSLTPooler }

procedure TServer.StopXSLTPooler;
begin { StopXSLTPooler }
  //Don't need reference counting because the server should get started once and left alone.
  //If RefCount=1 Then
    //Begin
      XSLTPooler.Free;
      XSLTPooler:=nil;
    //End;
  //Dec(RefCount);
end; { StopXSLTPooler }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                             TAtttitudeTireServer                             }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
function TXSLTransformServer.QueryServerParams(
  out OutValue: WideString): HResult;
begin { QueryServerParams }
  Try
    Result:=0;
    OutValue:='<?xml version="1.0"?><data type="server params"><cols>';
    OutValue:=OutValue+'<col><![CDATA[Pool size]]></col>';
    OutValue:=OutValue+'<col><![CDATA[Queue limit]]></col>';
    OutValue:=OutValue+'<col><![CDATA[Running]]></col>';
    OutValue:=OutValue+'</cols>';
    OutValue:=OutValue+'<rows>';
    OutValue:=OutValue+'<row>';
    OutValue:=OutValue+'<field name="Pool size"><![CDATA['+IntToStr(Server.XSLTPooler.PoolSize)+']]></field>';
    OutValue:=OutValue+'<field name="Queue limit"><![CDATA['+IntToStr(Server.XSLTPooler.QueueLimit)+']]></field>';
    If (Assigned(Server)) and (Server.XSLTPooler.Enabled) Then
      OutValue:=OutValue+'<field name="Running"><![CDATA[Yes]]></field>'
    Else
      OutValue:=OutValue+'<field name="Running"><![CDATA[No]]></field>';
    OutValue:=OutValue+'</row></rows></data>';
  Except
    On E:Exception Do
      Begin
        Result:=XSLTERROR;
        OutValue:='Exception (TXSLTransformServer.QueryServerParams):  '+E.Message;
      End;
  End;
end; { QueryServerParams }

function TXSLTransformServer.Transform(MaxWait: Integer; const XML,
  XSLT: WideString; out OutValue: WideString): HResult;
Var
  ObjRef:TPooledObjectRef;
begin { Transform }
  Try
    Result:=0;
    { Grab a transformation object }                                
    ObjRef:=Server.XSLTPooler.GrabObject(MaxWait);
    If ObjRef.GrabID<0 Then
      raise EXSLTransformServer.Create('Transform: '+Server.XSLTPooler.ParseError(ObjRef.GrabID));
    Try
      With TPooledXSLT(Server.XSLTPooler.Objects[ObjRef]) Do
        Begin
          if not XMLObj.LoadXML(XML) then
            raise Exception.Create('XML -- ' + XMLObj.parseError.reason);
          if not XSLObj.LoadXML(XSLT) then
            raise Exception.Create('XSL -- ' + XSLObj.parseError.reason);
          OutValue:=XMLObj.TransformNode(XSLObj);
        End;
    Finally
      Server.XSLTPooler.ReleaseObject(ObjRef);
    End;
  Except
    On E:Exception Do
      Begin
        OutValue:='Exception (TXSLTransformServer.Transform):  '+E.Message;
        Result:=XSLTERROR;
      End;
  End;
end; { Transform }

initialization
{==============================================================================}
{                                Initialization                                }
{==============================================================================}
  Try
    { Create TServer object }
    Server:=TServer.Create;
    CoInitializeEx(nil,COINIT_MULTITHREADED); //COM gets initialized by the the OmniHTTPd and IIS web servers, so this isn't called in the ISAPIs.  However, this is needed to keep the CreateOleObject calls from failing during autostart.
    If not(Server.AutostartServer) Then
      raise EXSLTransformServer.Create('Server refused to autostart.');
    { Class factory call }
    TTypedComObjectFactory.Create(ComServer, TXSLTransformServer, Class_XSLTransformServer,
      ciMultiInstance, tmFree);
  Except
    On E:Exception Do
      Begin
        ExceptionList:=TStringList.Create;
        Try
          ExceptionList.Add(DateTimeToStr(Now));
          ExceptionList.Add('Unable to autostart XSLTServer: '+E.Message);
          ExceptionList.SaveToFile(ExtractFilePath(Application.ExeName)+'xsltserver_startup_failure.txt');
        Finally
          ExceptionList.Free;
        End;
        raise;
      End;
  End;

finalization
{==============================================================================}
{                                 Finalization                                 }
{==============================================================================}
  Server.Free;
  CoUninitialize; //See note above (after CoInitializeEx() call).

end.
